<?php

namespace Application;

/**
 * Library class used for CSRF protection.
 */

use Application\Session;

class CsrfSecurity
{

    /**
     * Getting a CSRF token to add to a form
     * @return string
     */
    public static function getHiddenToken()
    {
        $csrfToken = self::getCSRFToken();
        return self::xssafe($csrfToken);
    }

    // xss mitigation functions
    public static function xssafe($data, $encoding = 'UTF-8')
    {
        return htmlspecialchars($data, ENT_QUOTES | ENT_HTML401, $encoding);
    }

    /**
     * Generate, and return the CSRF token
     * @return string
     */
    public static function getCSRFToken()
    {
        $token = bin2hex(openssl_random_pseudo_bytes(32));
        return $token;
    }

    public static function checkValidToken($token)
    {
        $isAntiCSRF = self::validateRequest($token);
        if (!$isAntiCSRF) {
            return false;
        }
        return true;
    }

    /**
     * Validate a request based on session
     * @return bool
     */
    public static function validateRequest($token)
    {
        if (!\is_string($token)) {
            return false;
        }
        $expected = Session::get('csrf_token');
        return \hash_equals($token, $expected);
    }

    /**
     * Checks a token validate request
     * If Development is on, then pass the validation
     * If not, attempt to validate the CSRF token
     *
     * @return bool
     */
    public static function checkRequest()
    {
        if (DEV_MODE) {
            return true;
        }
        $requestData = Request::form()->all();
        if (array_key_exists('csrf_token', $requestData)) {
            $token = $requestData['csrf_token'];
            if (self::checkValidToken($token)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns current token or generate a new csrf token if it does not exist in session
     */
    public static function getRefreshToken()
    {
        if (Session::has('csrf_token')) {
            $token = Session::get('csrf_token');
        } else {
            $token = CsrfSecurity::getCSRFToken();
            Session::set('csrf_token', $token);
        }
        return $token;
    }

    /**
     * Gets the CSRF token value
     * returns response for AJAX request in codebase.js
     */
    public function refreshCsrfToken()
    {
        $token = CsrfSecurity::getRefreshToken();
        Response::send(array(
            'success' => true,
            'data' => $token,
        ));
    }

    public function removeCsrfToken()
    {
        Session::remove('csrf_token');
    }
}
