<?php

namespace Application\Model;

require_once(dirname(dirname(__FILE__)).'/gateways/stripe/vendor/autoload.php');

use Application\CrmPayload;
use Application\CrmResponse;
use Application\Http;
use Application\Registry;
use Application\Response;
use Exception;
use Application\Request;

class Stripe extends BaseCrm
{

    protected $stripeObj;
    

    public function __construct($crmId)
    {
        parent::__construct($crmId);
    }

    protected function beforeAnyCrmClassMethodCall()
    {
        $this->params = $this->response = array();

        $this->params['secret'] = $this->password;
        $this->stripeObj = new \Stripe\StripeClient(
            $this->params['secret']
        );
    }

    protected function prospect($is_raw = false)
    {
        try {
            $this->params = array_replace($this->params, CrmPayload::all());
            $this->prepareShippingDetails();

            $payload['shipping']['address'] = $this->params['shipping'];
            $payload['shipping']['name']    = $this->params['name'];
            $payload['shipping']['phone']   = $this->params['phone'];
            $payload['name'] = $this->params['name'];
            $payload['phone'] = $this->params['phone'];
            $payload['email'] = $this->params['email'];

            $this->response = $this->stripeObj->customers->create($payload)->toArray();
            if(empty($this->response['id']))
            {
                return false;
            }
            if($is_raw){
                return $this->response['id'];
            }
        } catch (\Throwable $th) {
            CrmResponse::replace(array(
                'success' => false,
                'errors'  => array(
                    'crmError' => $th->getMessage(),
                ),
            ));
            return false;
        }

        CrmResponse::replace(array(
            'success'    => true,
            'prospectId' => $this->response['id'],
        ));
    }

    private function generateToken()
    {
        $cardInfo = $this->params['card'];
        try {
            $tokenResponse = $this->stripeObj->paymentMethods->create([
                'type' => 'card',  
                'card' => $cardInfo,
                'billing_details' => [
                    'address' => $this->params['billing'],
                    'name' => $this->params['billing_name'],
                    'email' => $this->params['email'],
                    'phone' => $this->params['phone']
                ]
                
            ])->toArray();
           
            $this->params['card_token'] = $tokenResponse['id'];
            $this->stripeObj->paymentMethods->attach(
                $tokenResponse['id'],
                ['customer' => $this->params['prospectId']]
            );
            return true;
        } catch (\Throwable $th) {
            CrmResponse::replace(array(
                'success' => false,
                'errors'  => array(
                    'crmError' => $th->getMessage(),
                ),
            ));
            return false;
        }
        
    }

    protected function newOrderWithProspect()
    {
        $this->params = array_replace($this->params, CrmPayload::all());
        $this->prepareShippingDetails();
        $this->prepareProductDetails();
        $this->prepareBillingDetails();
        $this->prepareCardDetails();
        if(!$this->generateToken()){
            return false;
        }


        $chargePayload['amount']                = $this->params['total'];
        $chargePayload['currency']              = 'usd';
        $chargePayload['payment_method_types']  = ['card'];
        $chargePayload['payment_method']        = $this->params['card_token'];
        $chargePayload['customer']              = $this->params['prospectId'];
        $chargePayload['receipt_email']         = $this->params['email'];
        $chargePayload['shipping']['address']   = $this->params['shipping'];
        $chargePayload['shipping']['name']      = $this->params['name'];
        $chargePayload['shipping']['phone']     = $this->params['phone'];
        $chargePayload['confirm']               = true;
        $chargePayload['description']           = $this->accessor->getValue($this->params, '[customNotes]');
        $chargePayload['metadata']              = $this->params['product_meta_data'];
        
        try {
            $this->response = $this->stripeObj->paymentIntents->create($chargePayload)->toArray();
        } catch (\Throwable $th) {
            CrmResponse::replace(array(
                'success' => false,
                'errors'  => array(
                    'crmError' => $th->getMessage(),
                ),
            ));
            return false;
        }

        unset($this->params['prospectId'], $this->params['userAgent']);

        if(empty($this->response['id']))
        {
            return false;
        }

        CrmResponse::replace(array(
            'success'           => true,
            'orderId'           => $this->response['id'],
            'customerId'        => $this->response['customer'],
            'payment_method'    => $this->response['payment_method'],
            'source'            => $this->response['source'],
            'receipt_url'       => $this->response['receipt_url'],
        ));
    }

    protected function newOrder()
    {
        $this->params = array_replace($this->params, CrmPayload::all());
        $prospectId = $this->prospect(true);
        $this->params['prospectId'] = $prospectId;
        if(empty($prospectId))
        {
            return false;
        }
        $this->prepareShippingDetails();
        $this->prepareProductDetails();
        $this->prepareBillingDetails();
        $this->prepareCardDetails();
        if(!$this->generateToken()){
            return false;
        }
        

        $chargePayload['amount']                = $this->params['total'];
        $chargePayload['currency']              = 'usd';
        $chargePayload['payment_method_types']  = ['card'];
        $chargePayload['payment_method']        = $this->params['card_token'];
        $chargePayload['customer']              = $this->params['prospectId'];
        $chargePayload['receipt_email']         = $this->params['email'];
        $chargePayload['shipping']['address']   = $this->params['shipping'];
        $chargePayload['shipping']['name']      = $this->params['name'];
        $chargePayload['shipping']['phone']     = $this->params['phone'];
        $chargePayload['confirm']               = true;
        $chargePayload['description']           = $this->accessor->getValue($this->params, '[customNotes]');
        $chargePayload['metadata']              = $this->params['product_meta_data'];
        
        try {
            $this->response = $this->stripeObj->paymentIntents->create($chargePayload)->toArray();

        } catch (\Throwable $th) {
            CrmResponse::replace(array(
                'success' => false,
                'errors'  => array(
                    'crmError' => $th->getMessage(),
                ),
            ));
            return false;
        }

        unset($this->params['prospectId'], $this->params['userAgent']);

        if(empty($this->response['id']))
        {
            return false;
        }

        CrmResponse::replace(array(
            'success'           => true,
            'orderId'           => $this->response['id'],
            'customerId'        => $this->response['customer'],
            'payment_method'    => $this->response['payment_method'],
            'source'            => $this->response['source'],
            'receipt_url'       => $this->response['receipt_url'],
        ));
    }

    protected function newOrderCardOnFile()
    {
        $this->params = array_replace($this->params, CrmPayload::all());
        $this->prepareShippingDetails();
        $this->prepareProductDetails();

        $chargePayload['amount']                = $this->params['total'];
        $chargePayload['currency']              = 'usd';
        $chargePayload['payment_method_types']  = ['card'];
        $chargePayload['payment_method']        = $this->params['payment_method'];
        $chargePayload['customer']              = $this->params['customerId'];
        $chargePayload['receipt_email']         = $this->params['email'];
        $chargePayload['shipping']['address']   = $this->params['shipping'];
        $chargePayload['shipping']['name']      = $this->params['name'];
        $chargePayload['shipping']['phone']     = $this->params['phone'];
        $chargePayload['confirm']               = true;
        $chargePayload['description']           = $this->accessor->getValue($this->params, '[customNotes]');
        $chargePayload['metadata']              = $this->params['product_meta_data'];

        try {
            $this->response = $this->stripeObj->paymentIntents->create($chargePayload)->toArray();
        } catch (\Throwable $th) {
            CrmResponse::replace(array(
                'success' => false,
                'errors'  => array(
                    'crmError' => $th->getMessage(),
                ),
            ));
            return false;
        }

        if(empty($this->response['id']))
        {
            return false;
        }

        CrmResponse::replace(array(
            'success'           => true,
            'orderId'           => $this->response['id'],
            'customerId'        => $this->response['customer'],
            'payment_method'    => $this->response['payment_method'],
            'source'            => $this->response['source'],
            'receipt_url'       => $this->response['receipt_url'],
        ));

    }

    private function makeHttpRequest()
    {
        return false;
    }

    public static function isValidCredential($credential)
    {
        $stripeObj = new \Stripe\StripeClient(
            $credential['password']
        );
        return $stripeObj->customers->all(['limit' => 2])->toArray();
    }

    private function prepareBillingDetails()
    {
        $billingDetails = array('billingSameAsShipping' => 'NO');
        if (strtolower($this->params['billingSameAsShipping']) === 'yes') {
            $billingDetails = $this->params['shipping'];
            $this->params['billing_name'] =  $this->params['name'];
        }
        else{
            $billingDetails = array(
                'line1'          => $this->accessor->getValue($this->params, '[billingAddress1]'),
                'line2'          => $this->accessor->getValue($this->params, '[billingAddress2]'),
                'postal_code'    => $this->accessor->getValue($this->params, '[billingZip]'),
                'city'           => $this->accessor->getValue($this->params, '[billingCity]'),
                'state'          => $this->accessor->getValue($this->params, '[billingState]'),
                'country'        => $this->accessor->getValue($this->params, '[billingCountry]'),
            );
            $this->params['billing_name'] =  $this->accessor->getValue($this->params, '[billingFirstName]') . " ". $this->accessor->getValue($this->params, '[billingLastName]');
        }
        
        $this->params['billing'] = $billingDetails;
    }

    private function prepareShippingDetails()
    {
        $shippingDetails = array(
            'line1'     => $this->accessor->getValue($this->params, '[shippingAddress1]'),
            'line2'     => $this->accessor->getValue($this->params, '[shippingAddress2]'),
            'postal_code'   => $this->accessor->getValue($this->params, '[shippingZip]'),
            'city'         => $this->accessor->getValue($this->params, '[shippingCity]'),
            'state'        => $this->accessor->getValue($this->params, '[shippingState]'),
            'country'      => $this->accessor->getValue($this->params, '[shippingCountry]'),
        );

        $this->params['shipping'] = $shippingDetails;
        $this->params['name'] = $this->params['firstName'].' '.$this->params['lastName'];
    }

    private function prepareCardDetails()
    {
        $cardDetails = array(
            'number'    => $this->accessor->getValue($this->params, '[cardNumber]'),
            'exp_month' => $this->accessor->getValue($this->params, '[cardExpiryMonth]'),
            'exp_year'  => !empty($this->params['cardExpiryYear']) ? '20'.$this->accessor->getValue($this->params, '[cardExpiryYear]') : '',
            'cvc'       => $this->accessor->getValue($this->params, '[cvv]')
        );
        
        $this->params['card'] = $cardDetails;

        unset(
            $this->params['cardExpiryMonth'], $this->params['cardExpiryYear'], $this->params['cvv']
        );
    }

    private function prepareProductDetails()
    {
        $result = array();
        $total = 0.00;
        $productMetaData = array();
        foreach ($this->params['products'] as $i => $product) {
            if (empty($product['productId'])) {
                continue;
            }
            $result[
                sprintf('product%s_id', $i + 1)
            ] = $this->accessor->getValue($product, '[productId]');
            $result[
                sprintf('product%s_qty', $i + 1)
            ] = $this->accessor->getValue($product, '[productQuantity]');
            $result[
                sprintf('product%s_price', $i + 1)
            ] = $this->accessor->getValue($product, '[productPrice]');
            $result[
                sprintf('product%s_shipPrice', $i + 1)
            ] = $this->accessor->getValue($product, '[shippingPrice]');

            $productMetaData[
                sprintf('product%s_id', $i + 1)
            ] = $this->accessor->getValue($product, '[productId]');

            $productMetaData[
                sprintf('product%s_name', $i + 1)
            ] = $this->accessor->getValue($product, '[campaign_lable]');

            if(!empty($this->params['isOrderVaultAPICall']))
            {
                $result[
                    sprintf('rebill%s_id', $i + 1)
                ] = $this->accessor->getValue($product, '[rebillId]');
            }
            $total = $total + ($this->accessor->getValue($product, '[productPrice]'));
            $shippingPrice = $this->accessor->getValue($product, '[shippingPrice]');
        }
        $this->params['product_meta_data'] = $productMetaData;
        
        if(!empty($this->params['stripe_shipping_price'])) {
            $shippingPrice = $this->params['stripe_shipping_price'];
            unset($this->params['stripe_shipping_price']);
        }

        $total = $total + $shippingPrice;
        $this->params['total'] = $total * 100;
        
        unset($this->params['products']);

        $this->params = array_replace($this->params, array_filter($result));
    }

    public function transactionQuery()
    {
        return;
    }

    public function orderView()
    {
        $this->beforeAnyCrmClassMethodCall();
        $customerId       = CrmPayload::get('orderIds')[0];
        try {
            $this->response = $this->stripeObj->paymentIntents->all(['customer' => $customerId])->toArray();
        } catch (\Throwable $th) {
            return $th->getMessage();
        }

        if(empty($this->response['data']))
        {
            return;
        }

        foreach ($this->response['data'] as $d) {
            if (!empty($d) && is_array($d)) {
                $result[$d['id']] = $d;
            } else {
                $result[0] = 'Not found!';
            }
        }

        CrmResponse::replace(array(
            'success' => true,
            'result'  => $result,
        ));
    }

}
